package cz.cvut.kbss.reporting.rest;

import com.fasterxml.jackson.core.type.TypeReference;
import cz.cvut.kbss.reporting.environment.Generator;
import cz.cvut.kbss.reporting.model.Tag;
import cz.cvut.kbss.reporting.rest.handler.RestExceptionHandler;
import cz.cvut.kbss.reporting.service.repository.TagService;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static org.hamcrest.CoreMatchers.containsString;
import static org.junit.Assert.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class TagControllerTest extends BaseControllerTestRunner {

    @Mock
    private TagService tagServiceMock;

    @InjectMocks
    private TagController controller;

    @Before
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        super.setUp();
        // This sets up only the single controller, no spring context is actually started
        // setControllerAdvice makes sure that our ExceptionHandler works
        this.mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new RestExceptionHandler())
                                      .build();
    }

    @Test
    public void findAllCallServiceToGetAllExistingTags() throws Exception {
        final List<Tag> tags = IntStream.range(0, 5).mapToObj(i -> Generator.generateTag("tag_" + i)).collect(
                Collectors.toList());
        when(tagServiceMock.findAll()).thenReturn(tags);

        final MvcResult mvcResult = mockMvc.perform(get("/tags")).andExpect(status().isOk()).andReturn();
        final List<Tag> result = readValue(mvcResult, new TypeReference<List<Tag>>() {
        });
        assertEquals(tags.size(), result.size());
        final Set<String> tagNames = tags.stream().map(Tag::getName).collect(Collectors.toSet());
        result.forEach(t -> assertTrue(tagNames.contains(t.getName())));
        verify(tagServiceMock).findAll();
    }

    @Test
    public void createTagPersistsTagAndReturnsLocationOfFindAll() throws Exception {
        final Tag tag = new Tag();
        tag.setName("newTag");
        final MvcResult mvcResult = mockMvc
                .perform(post("/tags").content(toJson(tag)).contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isCreated())
                .andReturn();
        final String location = mvcResult.getResponse().getHeader(HttpHeaders.LOCATION);
        assertNotNull(location);
        assertThat(location, containsString("/tags"));
        final ArgumentCaptor<Tag> captor = ArgumentCaptor.forClass(Tag.class);
        verify(tagServiceMock).persist(captor.capture());
        assertEquals(tag.getName(), captor.getValue().getName());
    }
}